import {
  ChangeDetectorRef,
  Component,
  EventEmitter,
  Inject,
  OnInit,
  Output,
  RendererFactory2,
  ViewChild,
  ViewEncapsulation
} from '@angular/core';
import {ActivatedRoute} from '@angular/router';
import {DOCUMENT} from "@angular/common";
import {Subscriber} from "rxjs";

interface addthis {
  layers: Refresh;
  init: Function;
  toolbox: Function;
}

interface Refresh {
  refresh: Function;
}

declare var addthis: addthis;

@Component({
  selector: 'addThis',
  template: `
<div id="addThis" class="addthis_inline_share_toolbox_lcx9_8cfy"></div>
<div *ngIf="showWarning" class="uk-alert uk-alert-warning uk-animation-fade" >
    Do the share buttons not appear? Please make sure, any blocking addon is disabled, and then reload the page.
</div>
  `
})
export class AddThisComponent implements OnInit {
  subs=[];
  showWarning = false;
  constructor(private route: ActivatedRoute, @Inject(DOCUMENT) private document, private rendererFactory: RendererFactory2) {}
  public ngOnDestroy() {
    for(let value of this.subs){
      if (value instanceof Subscriber) {
        value.unsubscribe();
      } else if (value instanceof Function) {
        value();
      }
    }
  }
  ngOnInit() {

    this.subs.push(this.route.queryParams.subscribe(data => {
      this.showWarning = false;
      try {
          if (!this.document.getElementById('addThisScript') && typeof document !== 'undefined') {
          // console.log(" create script AddThis");
          const renderer = this.rendererFactory.createRenderer(this.document, {
            id: '-1',
            encapsulation: ViewEncapsulation.None,
            styles: [],
            data: {}
          });
          const head = this.document.body;
          if (head === null) {
            throw new Error('<head> not found within DOCUMENT.');
          }
          const script = renderer.createElement('script');
          renderer.setAttribute(script, "id", "addThisScript");
          renderer.setAttribute(script, "src", "https://s7.addthis.com/js/300/addthis_widget.js#pubid=ra-559d24521cd8c080");
          renderer.setAttribute(script, "type", "text/javascript");
          renderer.appendChild(head, script);
        }
        if (typeof document !== 'undefined') {
          if(typeof addthis !== 'undefined' && addthis.layers && addthis.layers.refresh) {
            // console.log("Add This: Call Refresh")
            addthis.layers.refresh();
          }
        }
        this.subs.push(setTimeout(() => {
          if(this.document.getElementById('addThis') && !this.document.getElementById('addThis').innerText){
            this.showWarning = true;
          }
        }, 4000));
      }catch (e) {
          // console.error(e)
      }

    }));
  }
}
